
#include <windows.h>
#include "override.h"
#include "filter.h"
#include "settings.h"
#include "cache.h"

// enhancer patch
// see http://khdd.net/kanou/kangae/2006/Sep.html
#define adds(var, val, min, max)  ((var) = (var)+(val)<=(min) ? (min) : (var)+(val)>=(max) ? (max) : (var)+(val))
#define subs(var, val, min, max)  ((var) = (var)-(val)<=(min) ? (min) : (var)-(val)>=(max) ? (max) : (var)-(val))
#define add_R(var, val)		  adds((var), (val), min_R, max_R)
#define sub_R(var, val)		  subs((var), (val), min_R, max_R)
#define add_G(var, val)		  adds((var), (val), min_G, max_G)
#define sub_G(var, val)		  subs((var), (val), min_G, max_G)
#define add_B(var, val)		  adds((var), (val), min_B, max_B)
#define sub_B(var, val)		  adds((var), (val), min_B, max_B)

void EnhanceEdges(ETOCache* pCache, BYTE* lpPixels, int width, int height, BOOL opaque, COLORREF bgcolor, COLORREF textcolor)
{
	if (Settings::Enhance <= 0)
		return;
	else if (Settings::Enhance > 9)
		Settings::Enhance = 9;

	if (!opaque) {
		bgcolor = RGB(0xff,0xff,0xff);
		textcolor = RGB(0,0,0);
	}
	BYTE min_R = min(GetRValue(bgcolor), GetRValue(textcolor));
	BYTE max_R = max(GetRValue(bgcolor), GetRValue(textcolor));
	BYTE min_G = min(GetGValue(bgcolor), GetGValue(textcolor));
	BYTE max_G = max(GetGValue(bgcolor), GetGValue(textcolor));
	BYTE min_B = min(GetBValue(bgcolor), GetBValue(textcolor));
	BYTE max_B = max(GetBValue(bgcolor), GetBValue(textcolor));
	int  srcWidthBytes = pCache->Size().cx * 4;
	int  dstSkipBytes  = srcWidthBytes - (width*4 / Settings::Scale);
	int  w = (width*4 / Settings::Scale) + dstSkipBytes;
	register BYTE* dst;

	width  /= Settings::Scale;
	height /= Settings::Scale;
	dst = lpPixels + w;
	for(int y=1 ; y < height-1 ; y++) {
		register signed char* hdif = (signed char *)dst + srcWidthBytes * height;
		register signed char* vdif = hdif + srcWidthBytes * height;
		dst+=4; hdif+=4; vdif+=4;
		for (int x = 1 ; x < width-1 ; x++) {
			static const int coefficient[10] = { 0, 64, 48, 32, 24, 20, 16, 14, 12, 10 };
			int ratio = coefficient[Settings::Enhance];
			if (Settings::DontUseSubPixel) {
				hdif[0] = ((dst[0] << 1) - dst[-4]   - dst[4])   / ratio;
				hdif[1] = ((dst[1] << 1) - dst[-3]   - dst[5])   / ratio;
				hdif[2] = ((dst[2] << 1) - dst[-2]   - dst[6])   / ratio;
			}
			vdif[0] = ((dst[0] << 1) - dst[-w]   - dst[w])   / ratio;
			vdif[1] = ((dst[1] << 1) - dst[-w+1] - dst[w+1]) / ratio;
			vdif[2] = ((dst[2] << 1) - dst[-w+2] - dst[w+2]) / ratio;
			dst+=4; hdif+=4; vdif+=4;
		}
		dst += dstSkipBytes + 4;
	}
	dst = lpPixels + w;
	for(int y=1 ; y < height-1 ; y++) {
		register signed char* hdif = (signed char *)dst + srcWidthBytes * height;
		register signed char* vdif = hdif + srcWidthBytes * height;
		dst+=4; hdif+=4; vdif+=4;
		for (int x = 1 ; x < width-1 ; x++) {
			if (Settings::DontUseSubPixel) {
				sub_B(dst[-4], hdif[0] / 2);  add_B(dst[0], hdif[0]);  sub_B(dst[4],  hdif[0] / 2);
				sub_G(dst[-3], hdif[1] / 2);  add_G(dst[1], hdif[1]);  sub_G(dst[5],  hdif[1] / 2);
				sub_R(dst[-2], hdif[2] / 2);  add_R(dst[2], hdif[2]);  sub_R(dst[6],  hdif[2] / 2);
			}
			sub_B(dst[-w],  vdif[0] / 2);   add_B(dst[0], vdif[0]);   sub_B(dst[w],   vdif[0] / 2);
			sub_G(dst[1-w], vdif[1] / 2);   add_G(dst[1], vdif[1]);   sub_G(dst[w+1], vdif[1] / 2);
                        sub_R(dst[2-w], vdif[2] / 2);   add_R(dst[2], vdif[2]);   sub_R(dst[w+2], vdif[2] / 2);
			dst+=4; hdif+=4; vdif+=4;
		}
		dst += dstSkipBytes + 4;
	}
}
