#include "settings.h"

const UINT MAX_EXCLUDES = 32;
const UINT MAX_INDIVIDUALS = 32;

int Settings::Scale = 1;

UINT Settings::Quality = 0;
UINT Settings::Weight = 0;
UINT Settings::Enhance = 0;
UINT Settings::SubPixelDirection = 0;
float Settings::MaxHeight = 0.0f;

BOOL Settings::DontUseSubPixel = FALSE;
BOOL Settings::ForceAntialiasedQuality = FALSE;
BOOL Settings::IsWinXPorLater = FALSE;

StringHash *Settings::ExcludeList = NULL;
StringHashMod *Settings::ExcludeMod = NULL;
StringHash *Settings::IndividualList = NULL;

void Settings::Load(HMODULE hModule) {
	WCHAR szFileName[MAX_PATH + 1] = L"";
	// DLL̃tpX擾
	GetModuleFileName(hModule, szFileName, MAX_PATH);
	// gq".ini"ɒu
	lstrcpy(szFileName + lstrlen(szFileName) - 3, L"ini");
	// eݒǂݍ
	// INIt@C̗:
	// [General]
	// Quality=1
	// Weight=0
	// Enhance=0
	// UseSubPixel=0
	// SubPixelDirection=0
	// MaxHeight=0
	// ForceAntialiasedQuality=0
	Quality = GetPrivateProfileInt(L"General",L"Quality",2,szFileName);
	Weight = GetPrivateProfileInt(L"General",L"Weight",0,szFileName);
	Enhance = GetPrivateProfileInt(L"General",L"Enhance",0,szFileName);
	SubPixelDirection = GetPrivateProfileInt(L"General",L"SubPixelDirection",0,szFileName);
	MaxHeight = (float)GetPrivateProfileInt(L"General",L"MaxHeight",0,szFileName);
	
	DontUseSubPixel = ( GetPrivateProfileInt(L"General",L"UseSubPixel",0,szFileName) == 0 );
	ForceAntialiasedQuality = ( GetPrivateProfileInt(L"General",L"ForceAntialiasedQuality",0,szFileName) != 0 );

	// OS̃o[WXPȍ~ǂ
	OSVERSIONINFO osvi;
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx (&osvi);
	IsWinXPorLater = ((osvi.dwMajorVersion > 5) || ((osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion >= 1)));

	// quality
	if ( Quality < 0 ) { Quality = 0; }
	if ( Quality > 3 ) { Quality = 3; }
	Scale = Quality + 1;

	// Ɨ̈̊m
	const int cch = 65536;
	WCHAR* buffer = reinterpret_cast<WCHAR*>(HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(WCHAR) * cch));
	if (!buffer) {
		return;
	}

	// [Exclude]ZNV珜OtHgXgǂݍ
	delete[] ExcludeList;
	ExcludeList = new StringHash[MAX_EXCLUDES];
	ZeroMemory(buffer, sizeof(WCHAR) * cch);

	WCHAR* p = buffer;
	UINT   nb = 0;
	GetPrivateProfileSection(L"Exclude", p, cch, szFileName);
	
	while (*p != NULL) {
		bool flag = false;
		StringHash* exclist = ExcludeList;
		StringHash str(p);
		
		for (UINT i=0 ; i < nb ; i++) {
			if (exclist[i] == str) {
				flag = true;
				break;
			}
		}
		if (!flag) {
			if(nb < MAX_EXCLUDES) {
				exclist[nb++] = str;
			}
		}
		while(*p != NULL)  { p++; }
		p++;
	}
	
	// [Individual]ZNVʐݒtHgXgǂݍ
	delete[] IndividualList;
	IndividualList = new StringHash[MAX_INDIVIDUALS];
	ZeroMemory(buffer, sizeof(WCHAR) * cch);
	p = buffer;
	nb = 0;
	GetPrivateProfileSection(L"Individual", p, 4096, szFileName);
	while(*p) {
		bool b = false;
		StringHash* indlist = IndividualList;
		StringHash str(p);
		UINT weight = GetPrivateProfileInt(L"Individual", p, Weight, szFileName);
		for(UINT i=0 ; i < nb ; i++) {
			if(indlist[i] == str) {
				b = true;
				break;
			}
		}
		if(!b) {
			if(nb < MAX_INDIVIDUALS) {
				indlist[nb] = str;
				indlist[nb++].weight = weight;
			}
		}
		while(*p) p++;
		p++;
	}

	// [ExcludeModule]ZNVCWFNVȂvZXXgǂݍ
	delete[] ExcludeList;
	ExcludeMod = new StringHashMod[MAX_EXCLUDES];
	ZeroMemory(buffer, sizeof(WCHAR) * cch);
	
	p = buffer;
	nb = 0;
	GetPrivateProfileSection(L"ExcludeModule", p, cch, szFileName);
	while(*p) {
		bool b = false;
		StringHashMod* exclist = ExcludeMod;
		StringHashMod str(p);
		for(UINT i=0; i < nb; i++ ) {
			if(exclist[i] == str ) {
				b = true;
				break;
			}
		}
		if(!b) {
			if(nb < MAX_EXCLUDES) {
				exclist[nb++] = str;
			}
		}
		while(*p) p++;
		p++;
	}
	HeapFree(GetProcessHeap(), HEAP_NO_SERIALIZE, buffer);
}
BOOL Settings::UseScaling(void) {
	return Settings::Quality > 0;
}

BOOL Settings::IsFontIncluded(const LPCWSTR lpszFaceName) {
	StringHash* exclist = Settings::ExcludeList;
	StringHash str(lpszFaceName);
	for(UINT i=0 ; i < MAX_EXCLUDES && exclist[i].Hash() ; i++) {
		if(exclist[i] == str)
			return FALSE;
	}
	return TRUE;
}

BOOL Settings::IsFontIncluded(const LPCSTR lpszFaceName) {
	int bufferLength = MultiByteToWideChar(CP_ACP, 0, lpszFaceName, LF_FACESIZE, 0, 0);
	LPWSTR lpszUnicode = new wchar_t[bufferLength * 2];
	MultiByteToWideChar(CP_ACP, 0, lpszFaceName, LF_FACESIZE, lpszUnicode, bufferLength);
	BOOL b = IsFontIncluded(lpszUnicode);
	delete[] lpszUnicode;
	return b;
}

BOOL Settings::IsProcessExcluded()
{
	StringHashMod* exclist = ExcludeMod;
	for(UINT i=0; i < MAX_EXCLUDES && exclist[i].Hash(); i++ ) {
		if (GetModuleHandleW(exclist[i].c_str())) {
			return TRUE;
		}
	}
	return FALSE;
}


// LDCǂ`FbN
BOOL Settings::IsValidDC(const HDC hdc){
	if (GetDeviceCaps(hdc, TECHNOLOGY) != DT_RASDISPLAY) return FALSE;
	// ŃtHg`FbNs
	WCHAR szFaceName[LF_FACESIZE] = L"";
	GetTextFaceW(hdc, LF_FACESIZE, szFaceName);
	return IsFontIncluded(szFaceName);
}

// [Individual]ZNVɃEFCgw
UINT Settings::GetFontWeight(const LPCWSTR lpszFaceName){
	StringHash* indlist = IndividualList;
	StringHash str(lpszFaceName);
	for(UINT i=0 ; i < MAX_INDIVIDUALS && indlist[i].Hash() ; i++) {
		if(indlist[i] == str)
			return indlist[i].weight;
	}
	return Weight;
}